from _hyconfig.abc import AbstractModel
from _hycore.typefunc import alias


class ManagerBase:
    default: object

    def __init__(self):
        self.__data__ = {}

    def __setattr__(self, name, value):
        if isinstance(value, AbstractModel):
            self.__data__[name] = value
        else:
            super().__setattr__(name, value)

    def __getattr__(self, name):
        return self.__data__.get(name, self.default)

    def get(self, name):
        return self.__data__.get(name, self.default)

    def set(self, name, value):
        self.__data__[name] = value


class ModelManager(ManagerBase):
    models = alias['__data__'](mode=alias.mode.read_write)


class BackendManager(ManagerBase):
    backends = alias['__data__'](mode=alias.mode.read_write)




import dataclasses

from _hycore.utils import InstanceMapping
from _hycore.typefunc import iter_annotations


class ConfigItemMeta(type):
    def __init__(cls, name, bases, namespace):
        super().__init__(name, bases, namespace)
        cls._mapping = InstanceMapping()

    def __get__(self, instance, owner):
        if instance is None:
            return self

        if instance not in self._mapping:
            self._mapping[instance] = self()

        return self._mapping[instance]


class InlineConfig(metaclass=ConfigItemMeta):
    def __init_subclass__(cls, **kwargs):
        dataclasses.dataclass(cls)  # 使用 dataclasses 提供的功能实现结构体





from base64 import b64encode, b64decode

from ..abc.types import ConfigTypeBase, ConfigTypeMapping


class IntType(ConfigTypeBase, type=int): ...


class StringType(ConfigTypeBase, type=str): ...


class FloatType(ConfigTypeBase, type=float): ...


class BooleanType(ConfigTypeBase, type=bool): ...


class ListType(ConfigTypeBase, type=list): ...


class TupleType(ConfigTypeBase, type=tuple): ...


class DictType(ConfigTypeBase, type=dict): ...


class SetType(ConfigTypeBase, type=set): ...


class BytesType(ConfigTypeBase, types=bytes):
    def transform(self, value):
        if self.backend.support(bytes):
            return value
        else:
            return b64encode(value).decode()

    def load(self, value):
        if isinstance(value, str):
            return b64decode(value.encode())
        else:
            return value


builtin_type_mapping = m = ConfigTypeMapping()
m.add_types(IntType, StringType, FloatType, BooleanType, ListType, TupleType, DictType, SetType, BytesType)




from _hyconfig.config.manager import BackendManager, ModelManager


class ConfigError(Exception):
    ...


class HyConfig:
    """
    配置主类
    通过继承并添加ConfigItem类属性来定义配置结构
    """
    backends = ...
    models = ...

    def __init__(self):
        self.models = ModelManager()
        self.backends = BackendManager()




import json
from typing import Self

from ..abc import AbstractBackend, AbstractModel


class JsonBackend(AbstractBackend):

    def set_model(self, model: AbstractModel) -> Self:
        self.model = model
        return self

    def get_model(self) -> 'AbstractModel':
        return self.model

    def save(self, fd):
        json.dump(self.model.data, fd)

    def load(self, fd):
        self.model.init(json.load(fd))



import pickle
from ..abc import *

class PickleBackend(AbstractBackend):

    def set_model(self, model: 'AbstractModel') -> Self:
        self.model = model
        return self

    def get_model(self) -> 'AbstractModel':
        return self.model

    def save(self, fd):
        pickle.dump(self.model.data, fd)

    def load(self, fd):
        self.model.init(pickle.load(fd))



from abc import ABC
from typing import Any, Union


class ConfigTypeBase(ABC):
    parent: Any  # 配置项的父级, 通常为ConfigContainer
    type: Any  # 配置项值的类型

    def __init_subclass__(cls, *, type=None, **kwargs):
        cls.type = type

    def transform(self, value):  # 将配置项的值转换为后端可识别的配置数据
        ...

    def load(self, value):  # 将后端返回的配置数据加载到配置项中
        return value

    def validate(self, value) -> bool:  # 检查类型是否符合
        return isinstance(value, self.type)


_ConfigType = Union[ConfigTypeBase, type[ConfigTypeBase]]


class ConfigTypeMapping:
    def __init__(self):
        self._mapping = {}

    def add_pair(self, key_type, value_type):
        self._mapping[key_type] = value_type

    def add_type(self, config_type: _ConfigType):
        self.add_pair(config_type.type, config_type)

    def add_types(self, *config_types: _ConfigType):
        for config_type in config_types:
            self.add_type(config_type)

    def remove_pair(self, key_type):
        return self._mapping.pop(key_type)

    def remove_pairs(self, *key_types):
        for key_type in key_types:
            self.remove_pair(key_type)

    def remove_type(self, config_type: _ConfigType):
        return self.remove_pair(config_type.type)

    def get_type(self, config_type: _ConfigType):
        return self.get_pair(config_type.type)

    def get_pair(self, key_type):
        return self._mapping.get(key_type)

    def exists(self, key_type):
        return key_type in self._mapping

    def exists_type(self, config_type: _ConfigType):
        return self.exists(config_type.type)

    def __contains__(self, item):
        return self.exists(item)

    def __getitem__(self, item):
        return self.get_pair(item)

    def __setitem__(self, key, value):
        self.add_pair(key, value)

    def __delitem__(self, key):
        self.remove_pair(key)

    def __iter__(self):
        for k, v in self._mapping.items():
            yield k, v



from abc import ABC, abstractmethod


class AbstractModel(ABC):
    def __init__(self):
        self.data = None

    @abstractmethod
    def init(self, data):
        ...

    @abstractmethod
    def get(self, name, default=None):
        ...

    @abstractmethod
    def set(self, name, value):
        ...

    @abstractmethod
    def delete(self, name):
        ...

    @abstractmethod
    def exists(self, name):
        ...

    @abstractmethod
    def list(self):
        ...

    def modified(self):
        ...



import typing
from abc import ABC, abstractmethod
from typing import Self

if typing.TYPE_CHECKING:
    from .model import AbstractModel


class AbstractBackend(ABC):
    __support_types__ = None

    def __support_compare(self, type):
        return type in self.__support_types__

    def __support_check(self, type):
        for i in self.__support_types__:
            if issubclass(type, i):
                return True
        return False

    def __init_subclass__(cls, **kwargs):
        if cls.__support_types__ is None:
            raise TypeError("BackendABC subclass must have a support_types attribute.")

    def __class_getitem__(cls, item) -> 'AbstractBackend':
        return cls().set_model(item)

    def support(self, type):
        return self.__support_compare(type) or self.__support_check(type)

    @abstractmethod
    def set_model(self, model: 'AbstractModel') -> Self:
        ...

    @abstractmethod
    def get_model(self) -> 'AbstractModel':
        ...

    @abstractmethod
    def save(self, fd):
        ...

    @abstractmethod
    def load(self, fd):
        ...



from ..abc.model import *


class DefaultModel(AbstractModel):
    def __init__(self):
        super().__init__()
        self.data = {}
        self.changes = set()

    def init(self, data):
        self.data = data

    def set(self, name, value):
        if self.data.get(name) != value:
            self.changes.add(name)
        self.data[name] = value

    def get(self, name, default=None):
        return self.data.get(name, default)

    def delete(self, name):
        del self.data[name]
        self.changes.add(name)

    def exists(self, name):
        return name in self.data

    def list(self):
        return list(self.data.keys())

    def modified(self):
        ch = self.changes
        self.changes = set()
        return ch
